#include once "AxoclampDriver.bi"


sub CloseSession(byref hAXC as HAXC, IsOpen as boolean)
  if hAXC then
    if IsOpen then 
      dim as long nError
      AXC_CloseDevice(hAXC,@nError)
      if nError then
        var ErrorTxt = space(80)
        AXC_BuildErrorText(hAXC,nError,strptr(ErrorTxt),len(ErrorTxt))
        print "error in AXC_CloseDevice() code: " & nError & " error text: " & ErrorTxt
      else 
        print "device closed"
      end if
    end if
    AXC_DestroyHandle(hAXC) : hAXC = 0
    print "handle destroyed"
  end if
  print "session closed"
end sub

sub HandleError(hAXC as HAXC, sFunction as string, nError as long,IsOpen as boolean=false)
  if (hAXC)<>0 andalso (nError<>0) then
    var ErrorTxt = space(80)
    AXC_BuildErrorText(hAXC,nError,strptr(ErrorTxt),len(ErrorTxt))
    print "error in " & sFunction & " code: " & nError & " error text: " & ErrorTxt
  elseif (nError<>0) then
    print "error in " & sFunction & " code: " & nError
  else
    print "HandleError() warning: " & sFunction & " no handle no error code !"
  end if
  CloseSession(hAXC,IsOpen)
  beep : sleep : end 1
end sub

if AXC_CheckAPIVersion(AXC_APIVERSION_STR)=0 then
  print "error wrong API version " & AXC_APIVERSION_STR
  beep : sleep :end 1
end if  
print "API version " & AXC_APIVERSION_STR


dim as long bStartAsDemo = 1 ' !!! 0 in your case !!!
DIM as long nError
var hAXC = AXC_CreateHandle(bStartAsDemo,@nError)
if (hAXC=0) orelse (nError<>0) then HandleError(hAXC,"AXC_CreateHandle()",nError)

var SN = space(32)
var ok = AXC_GetSerialNumber(hAXC, strptr(SN), len(SN), @nError)
if (ok=0) orelse (nError<>0) then HandleError(hAXC,"AXC_GetSerialNumber()",nError)
print "serial number: " &  SN

' get serial number from first device (firstSN should match SN ? I don't know)
var firstSN=space(32)
ok = AXC_FindFirstDevice(hAXC, strptr(firstSN), len(firstSN), @nError)
if (ok=0) orelse (nError<>0) then HandleError(hAXC,"AXC_FindFirstDevice()",nError)
print "first device SN: " & firstSN
var nDevices=1


#if 0
' optional enumerate more devices (if any)
' of course here in demo mode I can use only one device (firstSN)
while (ok<>0) andalso (nError=0)
  var moreSN = space(32)
  ok = AXC_FindNextDevice(hAXC, strptr(moreSN), len(moreSN), @nError)
  if (ok<>0) andalso (nError=0) then
    nDevices += 1 : print "device: " & nDevices & " SN: " & moreSN
  end if  
wend
#endif
print "found: " & nDevices & " device" & iif(nDevices>1,"s","")

' open first device
var bReadHardware = 1
var IsOpen = AXC_OpenDevice(hAXC, strptr(firstSN), bReadHardware, @nError)
if (IsOpen=0) orelse (nError<>0) then HandleError(hAXC,"AXC_OpenDevice()",nError)

' It's redundant but why not
ok = AXC_IsDeviceOpen(hAXC, @IsOpen, @nError)
if (ok=0) orelse (nError<>0) then HandleError(hAXC,"AXC_IsDeviceOpen()",nError,IsOpen)

var deviceName = space(80)
ok = AXC_GetDeviceName(hAXC, strptr(DeviceName),len(DeviceName),@nError)
if (ok=0) orelse (nError<>0) then HandleError(hAXC,"AXC_GetDeviceName()",nError,IsOpen)
if len(DeviceName)<>80 then print "device name: " & DeviceName

var ManufactureDate = space(80)
ok=AXC_GetManufactureDate(hAXC, strptr(ManufactureDate),len(ManufactureDate),@nError)
if (ok=0) orelse (nError<>0) then HandleError(hAXC,"AXC_GetManufactureDate()",nError,IsOpen)
if len(ManufactureDate)<>80 then print "Manufacture Date: " & ManufactureDate

var IsDemo=0
ok=AXC_IsDemo(hAXC, @IsDemo, @nError)
if (ok=0) orelse (nError<>0) then HandleError(hAXC,"AXC_IsDemo()",nError,IsOpen)
print "first device is open" & iif(IsDemo," in DEMO mode!","")

' be sure the right mode is active
if (bStartAsDemo=0) andalso (IsDemo<>0) then
  print "warning: data acquisition are in DEMO mode !"
end if  


' now the fun part started :-)
screenres 640,480
dim as double minY=-.00001,maxY=.00001,oldminY,oldmaxY
dim as double xPos
dim as AXC_MeterData MeterData

while inkey()=""
  if (oldminY<>minY) orelse (oldmaxY<>maxY)  then
    oldminY=minY : oldmaxY=maxY
    window (0,maxY)-(639,minY)
    windowtitle " Y-Axis " & minY & " - "  & maxY
    cls:xPos=0:line (0,0)-step(639,0),7
  elseif xPos=640 then
    cls:xPos=0:line (0,0)-step(639,0),7
  end if  
  
  ok=AXC_AcquireMeterData(hAXC, @MeterData, @nError)
  if (ok=0) orelse (nError<>0) then HandleError(hAXC,"AXC_AcquireMeterData()",nError,IsOpen)


  var pValue = @MeterData.dMeter1
  ' using a pointer is simpler then 
  ' pset (xPos,MeterData.dMeter1),1
  ' pset (xPos,MeterData.dMeter2),2
  ' pset (xPos,MeterData.dMeter3),3
  ' pset (xPos,MeterData.dMeter4),4
  ' ...
  for i as integer = 0 to 3
    var y=pValue[i]*100
    pset(xPos,y),1+i
    if y<minY then minY=y
    if y>maxY then maxY=y
  next
  
  xPos+=1
  sleep 50
wend

' do it like a man :lol:
CloseSession(hAXC,IsOpen)

